package com.dsk.flink.dsc.sync;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.dsk.flink.dsc.common.function.AsyncMysqlDataTransferFunction;
import com.dsk.flink.dsc.common.sink.MysqlDataTransferSink;
import com.dsk.flink.dsc.utils.EnvProperties;
import com.dsk.flink.dsc.utils.EnvPropertiesUtil;
import com.dsk.flink.dsc.utils.EtlUtils;
import io.tidb.bigdata.tidb.JdbcConnectionProviderFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.restartstrategy.RestartStrategies;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.streaming.api.CheckpointingMode;
import org.apache.flink.streaming.api.datastream.AsyncDataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaConsumer;

import java.util.concurrent.TimeUnit;

/**
 * @author shezaixing
 * @date 2023/12/5 14:44
 * @description 同步至客户目标数据源任务
 *
 */
@Slf4j
public class SyncCustomerDataSource {

    public static void main(String[] args) throws Exception {

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setRestartStrategy(RestartStrategies.fixedDelayRestart(3, 30000));
        env.enableCheckpointing(60000);
        env.getCheckpointConfig().setCheckpointingMode(CheckpointingMode.EXACTLY_ONCE);
        env.getCheckpointConfig().setMinPauseBetweenCheckpoints(60000);
        env.getCheckpointConfig().setCheckpointTimeout(7200000);
        env.getCheckpointConfig().setMaxConcurrentCheckpoints(1);
        env.getCheckpointConfig().setExternalizedCheckpointCleanup(CheckpointConfig.ExternalizedCheckpointCleanup.RETAIN_ON_CANCELLATION);
        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(3);

        //获取用户自己的配置信息
        ParameterTool parameterTool = ParameterTool.fromArgs(args);
        String offsetTimestamp = parameterTool.get("offsetTimestamp");
        String propertiesPath = parameterTool.get("propertiesPath");
        EnvProperties envProps = EnvPropertiesUtil.getPropertiesFromArgsPath(propertiesPath);
        envProps.put("providerImpl", JdbcConnectionProviderFactory.HikariDataSourceJdbcConnectionProvider.class.getName());
        //System.out.println("读取到的配置文件：-> " + envProps.toString());

        FlinkKafkaConsumer<String> kafkaConsumer = new FlinkKafkaConsumer<String>(envProps.getKafka_topic(), new SimpleStringSchema(), EtlUtils.getKafkaConfig(envProps.getKafka_brokers(), envProps.getKafka_topic().concat("-group")));
        //lww添加
        if (StrUtil.isNotBlank(offsetTimestamp)) {
            kafkaConsumer.setStartFromTimestamp(Long.parseLong(offsetTimestamp));
        }

        SingleOutputStreamOperator<String> kafkaSource = env.addSource(kafkaConsumer)
                .setParallelism(1)
                .name("kafka-source")
                .uid("kafka-source");

        //kafkaSource.print("kafaka stream");

        SingleOutputStreamOperator<JSONObject> canalJsonStream = kafkaSource.map(JSONObject::parseObject)
                .name("canalJsonStream")
                .uid("canalJsonStream");

        //canalJsonStream.print("canal stream");

        SingleOutputStreamOperator<String> sqlResultStream = AsyncDataStream.orderedWait(canalJsonStream, new AsyncMysqlDataTransferFunction(envProps), 1200L, TimeUnit.SECONDS, 20)
                .filter(new FilterFunction<String>() {
                    @Override
                    public boolean filter(String value) throws Exception {
                        return StrUtil.isNotBlank(value) && !"err".equals(value);
                    }
                })
                .name("sqlResultStream")
                .uid("sqlResultStream");

        //sqlResultStream.print("sql result");

        sqlResultStream.addSink(new MysqlDataTransferSink(envProps)).name("sqlSinkStream").uid("sqlSinkStream");
        env.execute();
    }
}
